

/************************************************************************
 *									*
 *    This code has been developed and contributed by:			*
 *        Craig Barnes (EVL-UIC):					*
 *            moveXcursor()						*
 *									*
 *        Richard Gillilan (Cornell Univ) and Andy Pierce (IBM):	*
 *            lowerXwindow(), pushXbutton(), releaseXbutton()		*
 *									*
 *    Modified and documented for SIGGRAPH '94 VROOM by:		*
 *        Alan Verlo (EVL-UIC)						*
 *									*
 *    May 1994								*
 *    Nov 1994 - revised						*
 *									*
 ************************************************************************/


#include <stdio.h>
#include <math.h>


/*  for X mouse/wand events  */
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>



static char *program_name = NULL;
static char *display_name = NULL;
static int screen_num;
static Display *dpy;
static Screen *screen;
static Window root;
static XEvent event;
static XButtonEvent xb; 

/*************************************************
EXAMPLE
**************************************************
main()
{
openXdisplay();
moveXcursor(1280,1024);
closeXdisplay();
}
*************************************************/

/*=====   openXdisplay()  ====================================================*/

openXdisplay(void)
{
    dpy = XOpenDisplay(display_name);
    if (!dpy) {
        fprintf(stderr, "%s:  unable to open display '%s'\n",
                program_name, XDisplayName (display_name));
        exit (2);
    }

    screen_num = DefaultScreen(dpy);
    screen = DefaultScreenOfDisplay(dpy);
    root = RootWindow(dpy, screen_num);
}

/*=====   closeXdisplay() ====================================================*/

closeXdisplay(void)
{
    XCloseDisplay(dpy);
}

/*=====   moveXcursor()   ====================================================*/

moveXcursor(int x, int y)
{
    char *display_name = NULL;

    XWindowAttributes winatts;

    Window Rroot, Rchild, parent_win;
    Window Rparent, *Rchlst;

    int rc, Rrx, Rry, Rx, Ry;
    unsigned int Rmask, nchild;

    XWarpPointer(dpy,root,root,0,0,WidthOfScreen(screen),
                   HeightOfScreen(screen),x,y);
    XFlush(dpy);

}  /*  end moveXcursor()  */



/*=====   lowerXwindow()   ===================================================*/

lowerXwindow()
{

    char *display_name = NULL;

    XWindowAttributes winatts;

    Window Rroot, Rchild, parent_win;
    Window Rparent, *Rchlst;

    int rc,Rrx, Rry, Rx, Ry;
    unsigned int Rmask, nchild;

    event.type = ButtonPress;

    rc = XQueryPointer(dpy, root, &Rroot, &Rchild, &Rrx, 
                         &Rry, &Rx, &Ry, &Rmask);      

    XLowerWindow(dpy, Rchild); 

    XFlush(dpy);

}  /*  end lowerXwindow()  */



/*=====   pushXbutton()   ====================================================*/

pushXbutton()
{
    char *display_name = NULL;
    int x,y;

    XWindowAttributes winatts;

    Window Rroot, Rchild, parent_win;
    Window Rparent, *Rchlst;

    int rc, Rrx, Rry, Rx, Ry;
    unsigned int Rmask, nchild;

   /* fprintf(stderr, "    pushXbutton() ...\n"); */

    rc = XQueryPointer(dpy, root, &Rroot, &Rchild, &Rrx, 
                         &Rry, &Rx, &Ry, &Rmask);      

    while (Rchild != NULL) {

        parent_win = Rchild;

        XGetWindowAttributes(dpy,parent_win,&winatts);

        rc = XQueryPointer(dpy, parent_win, &Rroot, &Rchild, &Rrx, 
                         &Rry, &Rx, &Ry, &Rmask);      
    }

    /* propagate the XEvent back up the tree */

    if (winatts.all_event_masks & ButtonPressMask) {

            xb.type = ButtonPress;
   	    xb.window = parent_win;
	    xb.root =   root;
	    xb.subwindow = 0;
	    xb.x = Rx;
	    xb.y = Ry;
	    xb.x_root  = Rrx;
	    xb.y_root  = Rry;
	    xb.state = 0;
	    xb.button = Button1;
	    xb.same_screen = 1;

          rc = XSendEvent (dpy, PointerWindow, True, ButtonPressMask, (XEvent *) &xb);

          XFlush(dpy);        

    }  /*  end-if  */
  
    Rparent = parent_win;

    while (!(winatts.do_not_propagate_mask & ButtonPressMask)
               && (Rparent != NULL)) {

        XQueryTree(dpy, parent_win, &Rroot, &Rparent, &Rchlst, &nchild);

        if (Rparent != NULL) {

            parent_win = Rparent;
  
            XGetWindowAttributes(dpy, parent_win, &winatts);

            if (winatts.all_event_masks & ButtonPressMask) {

                rc = XQueryPointer(dpy, parent_win, &Rroot, &Rchild, &Rrx, 
                         &Rry, &Rx, &Ry, &Rmask);      

                xb.type = ButtonPress;
      	        xb.window = parent_win;
	        xb.root =   root;
	        xb.subwindow = 0;
	        xb.x = Rx;
	        xb.y = Ry;
	        xb.x_root  = Rrx;
	        xb.y_root  = Rry;
	        xb.state = 0;
	        xb.button = Button1;
	        xb.same_screen = 1;

                rc = XSendEvent (dpy, PointerWindow, True, ButtonPressMask, (XEvent *) &xb);

                XFlush(dpy);        

	    }  /*  end-if  */

        }  /*  end-if  */

    }  /*  end-while  */

}  /*  end pushXbutton()  */



/*=====   releaseXbutton()   =================================================*/

releaseXbutton()
{
    char *display_name = NULL;
    int x,y;

    XWindowAttributes winatts;

    Window Rroot, Rchild, parent_win;
    Window Rparent, *Rchlst;

    int rc, Rrx, Rry, Rx, Ry;
    unsigned int Rmask, nchild;

   /* fprintf(stderr, "    releaseXbutton() ...\n"); */

    rc = XQueryPointer(dpy, root, &Rroot, &Rchild, &Rrx, 
                         &Rry, &Rx, &Ry, &Rmask);      

    while (Rchild != NULL) {

         parent_win = Rchild;

         rc = XQueryPointer(dpy, parent_win, &Rroot, &Rchild, &Rrx, 
                         &Rry, &Rx, &Ry, &Rmask);      
    }

    /* propagate the XEvent back up the tree */

    XGetWindowAttributes(dpy, parent_win, &winatts);

    if (winatts.all_event_masks & ButtonReleaseMask) {

        xb.type = ButtonRelease;
        xb.window = parent_win;
	xb.root =   root;
	xb.subwindow = 0;
        xb.x = Rx;
        xb.y = Ry;
	xb.x_root  = Rrx;
	xb.y_root  = Rry;
	xb.state = 0;
	xb.button = Button1;

        rc = XSendEvent (dpy, PointerWindow, True, ButtonReleaseMask, (XEvent *) &xb);

        XFlush(dpy);        

    }  /*  end-if  */

    Rparent = parent_win;

    while (!(winatts.do_not_propagate_mask & ButtonReleaseMask) 
             && (Rparent !=  NULL)) {

        XQueryTree(dpy, parent_win, &Rroot, &Rparent, &Rchlst, &nchild);
              
        if (Rparent != NULL) {

            parent_win = Rparent;

            XGetWindowAttributes(dpy, parent_win, &winatts);

            if (winatts.all_event_masks & ButtonReleaseMask) {

                rc = XQueryPointer(dpy, parent_win, &Rroot, &Rchild, &Rrx, 
                            &Rry, &Rx, &Ry, &Rmask);      

                xb.type = ButtonRelease;
                xb.window = parent_win;
	        xb.root =   root;
	        xb.subwindow = 0;
	        xb.x = Rx;
	        xb.y = Ry;
	        xb.x_root  = Rrx;
	        xb.y_root  = Rry;
	        xb.state = 0;
                xb.button = Button1;
 
                rc = XSendEvent (dpy, PointerWindow, True, ButtonReleaseMask, (XEvent *) &xb);

                XFlush(dpy);        
		
	    }  /*  end-if  */

        }  /*  end-if  */

    }  /*  end-while  */

}  /*  end releaseXbutton()  */


